@@ -8,9 +8,11 @@ import numpy as np
 import matplotlib.pyplot as plt
 from scipy.interpolate import interp1d
 from scipy import signal
+from scipy.stats import ttest_1samp
 from nilearn import plotting, datasets, surface
 import nibabel as nib
-from neurora.stuff import get_affine, permutation_test, correct_by_threshold, get_bg_ch2, get_bg_ch2bet
+from neurora.stuff import get_affine, permutation_test, correct_by_threshold, get_bg_ch2, get_bg_ch2bet, \
+    clusterbased_permutation_1d_1samp_1sided, clusterbased_permutation_2d_1samp_1sided
 
 
 ' a function for plotting the RDM '
@@ -336,10 +338,13 @@ def plot_corrs_by_time(corrs, labels=None, time_unit=[0, 0.1]):
     return 0
 
 
-def plot_tbytsim_withstats(similarities, start_time=0, end_time=1, color='r', lim=[-0.1, 0.8]):
+' a function for plotting the time-by-time Similarities with statistical results'
+
+def plot_tbytsim_withstats(similarities, start_time=0, end_time=1, smooth=True, p=0.05, cbpt=True, color='r',
+                           lim=[-0.1, 0.8], figsize=[6.4, 3.6], x0=0, fontsize=16):
 
     """
-    Plot the time-by-time Similarities
+    Plot the time-by-time Similarities with statistical results
 
     Parameters
     ----------
@@ -351,10 +356,22 @@ def plot_tbytsim_withstats(similarities, start_time=0, end_time=1, color='r', li
         The start time.
     end_time : int or float. Default is 1.
         The end time.
+    smooth : bool True or False. Default is True.
+        Smooth the results or not.
+    p : float. Default is 0.05.
+        The threshold of p-values.
+    cbpt : bool True or False. Default is True.
+        Conduct cluster-based permutation test or not.
     color : matplotlib color or None. Default is 'r'.
         The color for the curve.
     lim : array or list [min, max]. Default is [-0.1, 0.8].
         The corrs view lims.
+    figsize : array or list, [size_X, size_Y]. Default is [6.4, 3.6].
+        The size of the figure.
+    x0 : float. Default is 0.
+        The Y-axis is at x=x0.
+    fontsize : int or float. Default is 16.
+        The fontsize of the labels.
     """
 
     if len(np.shape(similarities)) < 2 or len(np.shape(similarities)) > 3:
@@ -374,15 +391,16 @@ def plot_tbytsim_withstats(similarities, start_time=0, end_time=1, color='r', li
 
     tstep = float((end_time-start_time)/nts)
 
-    for sub in range(nsubs):
-        for t in range(nts):
+    if smooth is True:
+        for sub in range(nsubs):
+            for t in range(nts):
 
-            if t<=1:
-                similarities[sub, t] = np.average(similarities[sub, :t+3])
-            if t>1 and t<(nts-2):
-                similarities[sub, t] = np.average(similarities[sub, t-2:t+3])
-            if t>=(nts-2):
-                similarities[sub, t] = np.average(similarities[sub, t-2:])
+                if t<=1:
+                    similarities[sub, t] = np.average(similarities[sub, :t+3])
+                if t>1 and t<(nts-2):
+                    similarities[sub, t] = np.average(similarities[sub, t-2:t+3])
+                if t>=(nts-2):
+                    similarities[sub, t] = np.average(similarities[sub, t-2:])
 
     avg = np.average(similarities, axis=0)
     err = np.zeros([nts], dtype=np.float)
@@ -390,22 +408,33 @@ def plot_tbytsim_withstats(similarities, start_time=0, end_time=1, color='r', li
     for t in range(nts):
         err[t] = np.std(similarities[:, t], ddof=1)/np.sqrt(nsubs)
 
-    ps = np.zeros([nts], dtype=np.float)
-    chance = np.full([nsubs], 0)
+    if cbpt == True:
+        ps = clusterbased_permutation_1d_1samp_1sided(similarities, level=0, p_threshold=p)
+    else:
+        ps = np.zeros([nts])
+        for t in range(nts):
+            ps[t] = ttest_1samp(similarities[:, t], 0, alternative="greater")[1]
+            if ps[t] < p:
+                ps[t] = 1
+            else:
+                ps[t] = 0
 
     for t in range(nts):
-        ps[t] = permutation_test(similarities[:, t], chance)
-        if ps[t] < 0.05 and avg[t] > 0:
-            plt.plot(t*tstep+start_time, maxlim*0.9, 's', color=color, alpha=1)
+        if ps[t] == 1:
+            plt.plot(t*tstep+start_time, (maxlim-minlim)*0.9+minlim, 's', color=color, alpha=1)
             xi = [t*tstep+start_time, t*tstep+tstep+start_time]
             ymin = [0]
             ymax = [avg[t]-err[t]]
             plt.fill_between(xi, ymax, ymin, facecolor=color, alpha=0.1)
 
+    fig = plt.gcf()
+    fig.set_size_inches(figsize[0], figsize[1])
+
     ax = plt.gca()
     ax.spines["top"].set_visible(False)
     ax.spines["right"].set_visible(False)
     ax.spines["left"].set_linewidth(3)
+    ax.spines["left"].set_position(("data", x0))
     ax.spines["bottom"].set_linewidth(3)
     ax.spines['bottom'].set_position(('data', 0))
 
@@ -414,13 +443,201 @@ def plot_tbytsim_withstats(similarities, start_time=0, end_time=1, color='r', li
     plt.ylim(minlim, maxlim)
     plt.xlim(start_time, end_time)
     plt.tick_params(labelsize=12)
-    plt.xlabel("Time (s)", fontsize=16)
-    plt.ylabel("Representational Similarity", fontsize=16)
+    plt.xlabel("Time (s)", fontsize=fontsize)
+    plt.ylabel("Representational Similarity", fontsize=fontsize)
     plt.show()
 
     return 0
 
 
+' a function for plotting the time-by-time decoding accuracies '
+
+def plot_tbyt_decoding_acc(acc, start_time=0, end_time=1, time_interval=0.01, chance=0.5, p=0.05, cbpt=True, color='r',
+                           lim=[0.4, 0.8], figsize=[6.4, 3.6], x0=0, fontsize=16, avgshow=False):
+
+    """
+    Plot the time-by-time decoding accuracies
+
+    Parameters
+    ----------
+    acc : array
+        The decoding accuracies.
+        The size of acc should be [n_subs, n_ts]. n_subs, n_ts represent the number of subjects and number of
+        time-points.
+    start_time : int or float. Default is 0.
+        The start time.
+    end_time : int or float. Default is 1.
+        The end time.
+    time_interval : float. Default is 0.01.
+        The time interval between two time samples.
+    chance : float. Default is 0.5.
+        The chance level.
+    p : float. Default is 0.05.
+        The threshold of p-values.
+    cbpt : bool True or False. Default is True.
+        Conduct cluster-based permutation test or not.
+    color : matplotlib color or None. Default is 'r'.
+        The color for the curve.
+    lim : array or list [min, max]. Default is [0.4, 0.8].
+        The corrs view lims.
+    figsize : array or list, [size_X, size_Y]. Default is [6.4, 3.6].
+        The size of the figure.
+    x0 : float. Default is 0.
+        The Y-axis is at x=x0.
+    fontsize : int or float. Default is 16.
+        The fontsize of the labels.
+    avgshow : boolen True or False. Default is False.
+        Show the averaging decoding accuracies or not.
+    """
+
+    if len(np.shape(acc)) != 2:
+
+        return "Invalid input!"
+
+    nsubs, nts = np.shape(acc)
+    tstep = (end_time-start_time)/nts
+
+    if tstep != time_interval:
+
+        return "Invalid input!"
+
+    minlim = lim[0]
+    maxlim = lim[1]
+
+    avg = np.average(acc, axis=0)
+    err = np.zeros([nts])
+    for t in range(nts):
+        err[t] = np.std(acc[:, t], ddof=1) / np.sqrt(nsubs)
+
+    if cbpt == True:
+
+        ps = clusterbased_permutation_1d_1samp_1sided(acc, level=chance, p_threshold=p, iter=1000)
+
+    else:
+        ps = np.zeros([nts])
+        for t in range(nts):
+            ps[t] = ttest_1samp(acc[:, t], chance, alternative="greater")[1]
+            if ps[t] < p:
+                ps[t] = 1
+            else:
+                ps[t] = 0
+
+    for t in range(nts):
+        if ps[t] == 1:
+            plt.plot(t*tstep+start_time, (maxlim-minlim)*0.9+minlim, 's', color=color, alpha=0.8)
+            xi = [t*tstep+start_time, t*tstep+tstep+start_time]
+            ymin = [chance]
+            ymax = [avg[t] - err[t]]
+            plt.fill_between(xi, ymax, ymin, facecolor=color, alpha=0.2)
+
+    fig = plt.gcf()
+    fig.set_size_inches(figsize[0], figsize[1])
+    ax = plt.gca()
+    ax.spines["top"].set_visible(False)
+    ax.spines["right"].set_visible(False)
+    ax.spines["left"].set_linewidth(3)
+    ax.spines["left"].set_position(("data", x0))
+    ax.spines["bottom"].set_linewidth(3)
+    ax.spines["bottom"].set_position(("data", chance))
+    x = np.arange(start_time, end_time, tstep)
+    if avgshow is True:
+        plt.plot(x, avg, color=color, alpha=0.9)
+    plt.fill_between(x, avg+err, avg-err, facecolor=color, alpha=0.8)
+    plt.ylim(minlim, maxlim)
+    plt.xlim(start_time, end_time)
+    plt.tick_params(labelsize=12)
+    plt.xlabel("Time (s)", fontsize=fontsize)
+    plt.ylabel("Decoding Accuracy", fontsize=fontsize)
+    plt.show()
+
+
+' a function for plotting cross-temporal decoding accuracies '
+
+def plot_ct_decoding_acc(acc, start_timex=0, end_timex=1, start_timey=0, end_timey=1, chance=0.5, p=0.05, cbpt=True,
+                         lim=[0.4, 0.8], figsize=[6.4, 4.8], cmap="viridis", fontsize=16):
+
+    """
+    Plot the cross-temporal decoding accuracies
+
+    Parameters
+    ----------
+    acc : array
+        The decoding accuracies.
+        The size of acc should be [n_subs, n_tsx, n_tsy]. n_subs, n_tsx and n_tsy represent the number of subjects,
+        the number of training time-points and the number of test time-points.
+    start_timex : int or float. Default is 0.
+        The training start time.
+    end_timex : int or float. Default is 1.
+        The training end time.
+    start_timex : int or float. Default is 0.
+        The test start time.
+    end_timex : int or float. Default is 1.
+        The test end time.
+    chance : float. Default is 0.5.
+        The chance level.
+    p : float. Default is 0.05.
+        The threshold of p-values.
+    cbpt : bool True or False. Default is True.
+        Conduct cluster-based permutation test or not.
+    lim : array or list [min, max]. Default is [0.4, 0.8].
+        The corrs view lims.
+    figsize : array or list, [size_X, size_Y]. Default is [6.4, 3.6].
+        The size of the figure.
+    cmap :
+    fontsize : int or float. Default is 16.
+        The fontsize of the labels.
+    """
+
+    nsubs, nx, ny = np.shape(acc)
+    acc = np.transpose(acc, (0, 2, 1))
+    minlim = lim[0]
+    maxlim = lim[1]
+
+    tstepx = (end_timex-start_timex)/nx
+    tstepy = (end_timey-start_timey)/ny
+
+    if cbpt is True:
+
+        ps = clusterbased_permutation_2d_1samp_1sided(acc, level=chance, p_threshold=p, iter=1000)
+
+    else:
+        ps = np.zeros([nx, ny])
+        for t1 in range(nx):
+            for t2 in range(ny):
+                ps[t1, t2] = ttest_1samp(acc[:, t1, t2], chance, alternative="greater")[1]
+                if ps[t1, t2] < p:
+                    ps[t1, t2] = 1
+                else:
+                    ps[t1, t2] = 0
+
+    newps = np.zeros([nx+2, ny+2])
+    newps[1:nx+1, 1:nx+1] = ps
+    x = np.linspace(start_timex-0.5*tstepx, end_timex+0.5*tstepx, nx+2)
+    y = np.linspace(start_timey-0.5*tstepy, end_timey+0.5*tstepy, ny+2)
+    X, Y = np.meshgrid(x, y)
+    plt.contour(X, Y, newps, [0, 1], colors="silver", alpha=0.9, linewidths=3, linestyles="dashed")
+
+    fig = plt.gcf()
+    fig.set_size_inches(figsize[0], figsize[1])
+    ax = plt.gca()
+    ax.spines["top"].set_visible(False)
+    ax.spines["right"].set_visible(False)
+    ax.spines["left"].set_linewidth(2)
+    ax.spines["bottom"].set_linewidth(2)
+    avg = np.average(acc, axis=0)
+    plt.imshow(avg, extent=(start_timex, end_timex, start_timey, end_timey), cmap=cmap, origin="lower", clim=(minlim, maxlim))
+    cb = plt.colorbar()
+    cb.ax.tick_params(labelsize=12)
+    font = {'size': 15}
+    cb.set_label("Decoding Accuracy", fontdict=font)
+    plt.xlim(start_timex, end_timex)
+    plt.ylim(start_timey, end_timey)
+    plt.tick_params(labelsize=12)
+    plt.xlabel("Training Time (s)", fontsize=fontsize)
+    plt.ylabel("Test Time (s)", fontsize=fontsize)
+    plt.show()
+
+
 ' a function for plotting the hotmap of correlations coefficients for channels/regions by time sequence '
 
 def plot_corrs_hotmap(corrs, chllabels=None, time_unit=[0, 0.1], lim=[0, 1], smooth=False, figsize=None, cmap=None):
